@@ -41,12 +41,11 @@ void debug( int level, char *fmt, ... )
 {
     static char debug_buffer[HUGE_BUFFER];
     va_list argptr;
-    int cnt;
     //printf( "debug call\n" );
     if( level != DBG_LEVEL ) 
         return;
     va_start(argptr, fmt);
-    cnt = vsprintf(debug_buffer, fmt, argptr);
+    vsprintf(debug_buffer, fmt, argptr);
     va_end(argptr);
     fprintf( stderr, "DBG %d: %s\n", level, debug_buffer );
     fflush( stderr);
@@ -123,6 +122,10 @@ namelist parse_class_names(const char *s) {
     token = strtok (cp, delim);
     while ( token != NULL ) {
         namenode *tmp = NEW (namenode);
+        if (tmp == NULL) {
+            fprintf(stderr, "Out of memory\n");
+            exit(1);
+        }
         tmp->name = strdup(token);
         if (tmp->name == NULL) {
             fprintf(stderr, "Out of memory\n");
@@ -233,23 +236,24 @@ umlattrlist copy_attributes(umlattrlist src)
  */
 char *create_package_dir( const batch *batch, umlpackage *pkg )
 {
-    char * fulldirname, *dirname;
-    int ret;
+    char *fulldirname, *dirname, fulldirnamedup[BIG_BUFFER];
     /* created directories permissions */
     mode_t dir_mask = S_IRUSR | S_IWUSR | S_IXUSR |S_IRGRP | S_IXGRP;
-    if( pkg == NULL ) {
+    if (pkg == NULL) {
         return NULL;
     }
-    if( (batch->buildtree == 0) || (pkg->name == NULL )) {
+    if (batch->buildtree == 0 || pkg->name == NULL) {
         pkg->directory = batch->outdir;
     } else {
-        fulldirname = strdup( batch->outdir );
+        fulldirname = (char*)my_malloc(BIG_BUFFER);
+        sprintf(fulldirname, "%s", batch->outdir);
         dirname = strdup(pkg->name);
         dirname = strtok( dirname, "." );
-        while( dirname != NULL ) {
-            sprintf( fulldirname, "%s/%s", fulldirname, dirname );
-            /* TODO : should create only if not existant */
-            ret = mkdir( fulldirname, dir_mask );
+        while (dirname != NULL) {
+            sprintf( fulldirnamedup, "%s/%s", fulldirname, dirname );
+            sprintf( fulldirname, "%s", fulldirnamedup );
+            /* TODO : should create only if not existent */
+            mkdir( fulldirname, dir_mask );
             dirname = strtok( NULL, "." );
         }
         /* set the package directory used later for source file creation */
@@ -717,6 +721,7 @@ int d2c_backup(char *filename)
             if (errno != ENOENT)
             {
                 fprintf(stderr, "Error %d while trying to delete file %s\n", errno, backup_filename);
+                free(backup_filename);
                 return -1;
             }
         }
@@ -725,10 +730,12 @@ int d2c_backup(char *filename)
             if (errno != ENOENT)
             {
                 fprintf(stderr, "Error %d while trying to rename %s to %s\n", errno, filename, backup_filename);
+                free(backup_filename);
                 return -1;
             }
         }
     }
+    free(backup_filename);
     return 0;
 }
 
@@ -844,11 +851,10 @@ int _d2c_fprintf(FILE *f, char *fmt, ...)
 int d2c_directprintf(FILE *f, char *fmt, ...)
 {
     va_list argptr;
-    int cnt;
 
     d2c_indentate(f);
     va_start(argptr, fmt);
-    cnt = vfprintf(f, fmt, argptr);
+    vfprintf(f, fmt, argptr);
     va_end(argptr);
     return 0;
 }
