@@ -34,13 +34,13 @@ static batch *gb;   /* The current batch being processed.  */
 /* Utilities.  */
 
 static void
-check_umlattr (umlattribute *u, char *typename)
+check_umlattr (umlattribute *u, char *typename_)
 {
     /* Check settings that don't make sense for Ada generation.  */
     if (u->visibility == '1')
-        fprintf (stderr, "%s/%s: ignoring non-visibility\n", typename, u->name);
+        fprintf (stderr, "%s/%s: ignoring non-visibility\n", typename_, u->name);
     if (u->isstatic)
-        fprintf (stderr, "%s/%s: ignoring staticness\n", typename, u->name);
+        fprintf (stderr, "%s/%s: ignoring staticness\n", typename_, u->name);
 }
 
 static char *
@@ -214,8 +214,7 @@ gen_static_attributes (umlattrlist umla, int do_private)
         umlclassnode *ref;
 
         if (!umla->key.isstatic ||
-            (umla->key.visibility == '0' && do_private ||
-             umla->key.visibility != '0' && !do_private)) {
+            (umla->key.visibility != '0' && !do_private)) {
             umla = umla->next;
             continue;
         }
@@ -256,6 +255,7 @@ gen_static_attributes (umlattrlist umla, int do_private)
                 emit ("%s", adaname (umla->key.type));
             emit (";\n\n");
         }
+        umla = umla->next;
     }
     return number_of_static_attributes;
 }
@@ -664,10 +664,10 @@ generate_code_ada (batch *b)
 
         /* add license to the header */
         if (b->license != NULL) {
-            char lc;
+            int lc;
             rewind (licensefile);
             while ((lc = fgetc (licensefile)) != EOF)
-                print ("%c", lc);
+                print ("%c", (char) lc);
         }
 
         includes = NULL;
