@@ -3,7 +3,7 @@
                              -------------------
     begin                : Sat Dec 16 2000
     copyright            : (C) 2000-2001 by Javier O'Hara
-                           (C) 2002 by Oliver Kellogg
+                           (C) 2002-2014 by Oliver Kellogg
     email                : joh314@users.sourceforge.net
                            okellogg@users.sourceforge.net
  ***************************************************************************/
@@ -68,7 +68,6 @@ pass_by_reference (umlclass *cl)
     if (strlen (st) == 0)
         return 1;
     if (is_typedef_stereo (st)) {
-        umlattrlist umla = cl->attributes;
         umlclassnode *ref = find_by_name (gb->classlist, cl->name);
         if (ref == NULL)
             return 0;
@@ -244,13 +243,16 @@ gen_class (umlclassnode *node)
         */
         while (assoc != NULL) {
             umlclassnode *ref;
-            ref = find_by_name (gb->classlist, assoc->key->name);
-            print ("");
-            if (ref != NULL)
-                emit ("%s", fqname (ref, !assoc->composite));
-            else
-                emit ("%s", cppname (assoc->key->name));
-            emit (" %s;\n", assoc->name);
+            if (assoc->name[0] != '\0')
+            {
+                ref = find_by_name (gb->classlist, assoc->key->name);
+                print ("");
+                if (ref != NULL)
+                    emit ("%s", fqname (ref, !assoc->composite));
+                else
+                    emit ("%s", cppname (assoc->key->name));
+                emit (" %s;\n", assoc->name);
+            }
             assoc = assoc->next;
         }
     }
@@ -323,7 +325,7 @@ gen_class (umlclassnode *node)
             indentlevel++;
         }
         while (umlo != NULL) {
-            umlattrlist tmpa = umlo->key.parameters;
+            umlattrlist tmpa;
             if (is_valuetype) {
                 if (umlo->key.attr.visibility != '0')
                     fprintf (stderr, "CORBAValue %s/%s: must be public\n",
@@ -383,9 +385,7 @@ gen_class (umlclassnode *node)
             if (umlo->key.attr.value[0]) {
                 // virtual
                 if ((umlo->key.attr.isabstract || is_valuetype) &&
-                    (umlo->key.attr.name[0] == '~'))
-                    emit (" {}");
-                else
+                    umlo->key.attr.name[0] != '~')
                     emit (" = %s", umlo->key.attr.value);
             }
             emit (";\n");
@@ -612,10 +612,10 @@ generate_code_cpp (batch *b)
 
         /* add license to the header */
         if (b->license != NULL) {
-            char lc;
+            int lc;
             rewind (licensefile);
             while ((lc = fgetc (licensefile)) != EOF)
-                print ("%c", lc);
+                print ("%c", (char) lc);
         }
 
         includes = NULL;
