@@ -85,10 +85,10 @@ void generate_code_ruby(batch *b) {
 /* header */
                 /* add license to the header */
                 if (b->license != NULL) {
-                    char lc;
+                    int lc;
                     rewind(licensefile);
-                    while((lc = fgetc(licensefile)) != EOF){
-                        fprintf(outfile,"%c",lc);
+                    while ((lc = fgetc(licensefile)) != EOF) {
+                        fprintf(outfile,"%c", (char) lc);
                     }
                 }
 
@@ -174,7 +174,7 @@ void generate_code_ruby(batch *b) {
                     if ( strlen(tmplist->key->comment) > 0 ) {
                         fprintf(outfile,"# %s\n",tmplist->key->comment);
                     } else {
-                        fprintf(outfile,"# XXX\n",tmplist->key->comment);
+                        fprintf(outfile,"# XXX\n");
                     }
 
                     if (tmplist->key->isabstract) {
@@ -382,3 +382,5 @@ void generate_code_ruby(batch *b) {
         tmplist = tmplist->next;
     }
 }
+
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
