@@ -1,6 +1,6 @@
 /***************************************************************************
                           generate_code_shp.c  - generate batch file for
-                                                 shapefile creation 
+                                                 shapefile creation
                              -------------------
     begin                : Tue Oct 16 2001
     copyright            : (C) by Steffen Macke
@@ -21,18 +21,9 @@
 #include "dia2code.h"
 
 void generate_code_shp(batch *b) {
-    /*
-    umlclasslist tmplist,parents,dependencies;
-    umlassoclist associations;
-    namelist used_classes,tmpnamelist;
-    int tmpv;
-    umlattrlist umla,tmpa;
-    umloplist umlo;
-    */
     umlclasslist tmplist;
     umlclasslist parentlist, parentlist2;
     umlattrlist umla;
-    int tmpv;
     char *tmpname;
     char outfilename[BIG_BUFFER];
     FILE * outfileshp, *dummyfile;
@@ -79,8 +70,6 @@ void generate_code_shp(batch *b) {
 
                 free(tmpname);
 
-                tmpv = -1;
-  
                 if(tmplist->key->isabstract == 0) {
 
                     /* create attribute table */
@@ -92,10 +81,10 @@ void generate_code_shp(batch *b) {
                         if((strcmp(umla->key.name,"Shape") != 0)&&
                            (umla->key.visibility != 1)) {
                             if(strcmp(umla->key.type,"String") == 0) {
-                                fprintf(outfileshp, " -s %s 255", 
+                                fprintf(outfileshp, " -s %s 255",
                                         umla->key.name);
                             }
-                            else if((strcmp(umla->key.type, 
+                            else if((strcmp(umla->key.type,
                                            "CodedValue") == 0)||
                                 strcmp(umla->key.type, "Integer") ==0 ) {
                                 fprintf(outfileshp, " -n %s 16 0",
@@ -105,7 +94,7 @@ void generate_code_shp(batch *b) {
                                 fprintf(outfileshp, " -n %s 16 3",
                                         umla->key.name);
                             }
-                        }                
+                        }
                         umla = umla->next;
 
                         if((umla == NULL)&&(parentlist != NULL)) {
@@ -113,7 +102,7 @@ void generate_code_shp(batch *b) {
                             if(parentlist != NULL) {
                                 umla = parentlist->key->attributes;
                                 parentlist2 = b->classlist;
-                                while((strcmp(parentlist->key->name, 
+                                while((strcmp(parentlist->key->name,
                                               parentlist2->key->name) != 0)&&
                                       (parentlist2 != NULL))
                                     parentlist2 = parentlist2->next;
@@ -123,18 +112,18 @@ void generate_code_shp(batch *b) {
 
                     }
                     fprintf(outfileshp, "\n");
-                    
+
                     /* create shp file */
                     umla = tmplist->key->attributes;
                        parentlist = tmplist;
                     while ( umla != NULL) {
                         if(strcmp(umla->key.name,"Shape") == 0) {
-                            if(strcmp(strtolower(umla->key.type), 
+                            if(strcmp(strtolower(umla->key.type),
                                       "polyline") == 0) {
                                 strcpy(umla->key.type, "arc");
                             }
-                            fprintf(outfileshp, "shpcreate %s %s\n\n", 
-                                    tmplist->key->name, 
+                            fprintf(outfileshp, "shpcreate %s %s\n\n",
+                                    tmplist->key->name,
                                     strtolower(umla->key.type));
                             break;
                         }
@@ -145,7 +134,7 @@ void generate_code_shp(batch *b) {
                             if(parentlist != NULL) {
                                 umla = parentlist->key->attributes;
                                 parentlist2 = b->classlist;
-                                while((strcmp(parentlist->key->name, 
+                                while((strcmp(parentlist->key->name,
                                               parentlist2->key->name) != 0)&&
                                       (parentlist2 != NULL))
                                     parentlist2 = parentlist2->next;
@@ -155,10 +144,11 @@ void generate_code_shp(batch *b) {
 
                     }
                 }
-            }            
-            fclose(outfileshp);
+                fclose(outfileshp);
+            }
         }
         tmplist = tmplist->next;
     }
 }
 
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
