@@ -17,6 +17,10 @@
 
 #include "parse_diagram.h"
 
+#ifndef MIN
+#define MIN(x, y) (x < y ? x : y)
+#endif
+
 /* In case of unnamed associations, the attribute will get the name
    "unnamed_" followed by the anon_cnt converted to string:  */
 static unsigned anon_cnt = 0;
@@ -45,7 +49,7 @@ void parse_dia_string(xmlNodePtr stringnode, char *buffer) {
     xmlChar *content;
 
     content = xmlNodeGetContent(stringnode);
-    if ( sscanf(content, sscanfmt(), buffer) == 0) {
+    if (sscanf(content, sscanfmt(), buffer) == 0) {
         buffer[0] = 0;
     }
     free(content);
@@ -57,10 +61,7 @@ void parse_dia_string_large(xmlNodePtr stringnode, char *buffer) {
     char fmt[16];
 
     content = xmlNodeGetContent(stringnode);
-    sprintf (fmt, "#%%%d[^#]#", LARGE_BUFFER - 1);
-    if ( sscanf(content, fmt, buffer) == 0) {
-        buffer[0] = 0;
-    }
+    strncpy (buffer, content + 1, MIN(strlen(content) - 2, LARGE_BUFFER - 1));
     free(content);
 }
 
@@ -81,7 +82,7 @@ int parse_boolean(xmlNodePtr booleannode) {
 
 void addparent(umlclasslist base, umlclasslist derived) {
     umlclasslist tmp;
-    tmp = (umlclasslist) my_malloc ( sizeof(umlclassnode) );
+    tmp = NEW (umlclassnode);
     tmp->key = base->key;
     tmp->next = derived->parents;
     derived->parents = tmp;
@@ -89,25 +90,22 @@ void addparent(umlclasslist base, umlclasslist derived) {
 
 void adddependency(umlclasslist dependent, umlclasslist dependee) {
     umlclasslist tmp;
-    tmp = (umlclasslist) my_malloc ( sizeof(umlclassnode) );
+    tmp = NEW (umlclassnode);
     tmp->key = dependent->key;
     tmp->next = dependee->dependencies;
     dependee->dependencies = tmp;
 }
 
-void addaggregate(char * name, char composite, umlclasslist base, umlclasslist associate) {
+void addaggregate(char *name, char composite, umlclasslist base,
+                  umlclasslist associate, char *multiplicity) {
     umlassoclist tmp;
-    tmp = (umlassoclist) my_malloc ( sizeof(umlassocnode) );
-    if (name != NULL) {
+    tmp = NEW (umlassocnode);
+    if (name != NULL && strlen (name) > 2)
         sscanf(name, sscanfmt(), tmp->name);
-        if (tmp->name[0] == '\0') {
-            ++anon_cnt;
-            sprintf (tmp->name, "unnamed_%d", anon_cnt);
-        }
-    } else {
-        printf("warning: unnamed association between %s and %s\n", base->key->name, associate->key->name);
-        strcpy(tmp->name, "unnamed");
-    }
+    if (multiplicity != NULL)
+        sscanf(multiplicity, sscanfmt(), tmp->multiplicity);
+    else
+        sprintf(tmp->multiplicity, "1");
     tmp->key = base->key;
     tmp->composite = composite;
     tmp->next = associate->associations;
@@ -123,12 +121,13 @@ void inherit_realize ( umlclasslist classlist, char * base, char * derived ) {
     }
 }
 
-void associate ( umlclasslist classlist, char * name, char composite, char * base, char * aggregate) {
+void associate ( umlclasslist classlist, char * name, char composite,
+                 char * base, char * aggregate, char *multiplicity) {
     umlclasslist umlbase, umlaggregate;
     umlbase = find(classlist, base);
     umlaggregate = find(classlist, aggregate);
     if ( umlbase != NULL && umlaggregate != NULL) {
-        addaggregate(name, composite, umlbase, umlaggregate);
+        addaggregate(name, composite, umlbase, umlaggregate, multiplicity);
     }
 }
 
@@ -222,7 +221,9 @@ void parse_attribute(xmlNodePtr node, umlattribute *tmp) {
             free(attrval);
         } else if ( eq("abstract", nodename)) {
             tmp->isabstract = parse_boolean(node->xmlChildrenNode);
-        } else if ( eq("class_scope", nodename)) {
+        } else if ( eq("class_scope", nodename) ||
+                    eq("primary_key", nodename)) {
+            // the SQL code generator defines static attributes as primary keys
             tmp->isstatic = parse_boolean(node->xmlChildrenNode);
         } else if ( eq("query", nodename)) {
             tmp->isconstant = parse_boolean(node->xmlChildrenNode);
@@ -235,7 +236,7 @@ void parse_attribute(xmlNodePtr node, umlattribute *tmp) {
 umlattrlist parse_attributes(xmlNodePtr node) {
     umlattrlist list = NULL, an;
     while ( node != NULL ) {
-        an = (umlattrlist) my_malloc(sizeof(umlattrnode));
+        an = NEW (umlattrnode);
         an->next = NULL;
         parse_attribute(node->xmlChildrenNode, &(an->key));
         list = insert_attribute(an, list);
@@ -260,7 +261,7 @@ void parse_operation(xmlNodePtr node, umloperation *tmp) {
 umloplist parse_operations(xmlNodePtr node) {
     umloplist list = NULL, on;
     while ( node != NULL ) {
-        on = (umlopnode*) my_malloc(sizeof(umlopnode));
+        on = NEW (umlopnode);
         on->next = NULL;
         on->key.implementation = NULL;
         parse_operation(node->xmlChildrenNode, &(on->key));
@@ -280,8 +281,7 @@ umltemplatelist parse_templates(xmlNodePtr node) {
     while ( node != NULL) {
         if ( node->xmlChildrenNode->xmlChildrenNode->xmlChildrenNode != NULL &&
                 node->xmlChildrenNode->next->xmlChildrenNode->xmlChildrenNode != NULL ) {
-
-            tn = (umltemplatenode*) my_malloc(sizeof(umltemplatenode));
+            tn = NEW (umltemplatenode);
             tn->next = NULL;
             parse_template(node->xmlChildrenNode, &(tn->key));
             list = insert_template(tn, list);
@@ -304,8 +304,8 @@ void make_javabean_methods(umlclass *myself) {
     while (attrlist != NULL) {
         if ( ! attrlist->key.isabstract) {
             /* The SET method */
-            operation = (umloplist) my_malloc (sizeof(umlopnode));
-            parameter = (umlattrlist) my_malloc (sizeof(umlattrnode));
+            operation = NEW (umlopnode);
+            parameter = NEW (umlattrnode);
 
             sprintf(parameter->key.name, "value");
             strncpy(parameter->key.type, attrlist->key.type, SMALL_BUFFER);
@@ -317,14 +317,14 @@ void make_javabean_methods(umlclass *myself) {
             parameter->next = NULL;
             operation->key.parameters = parameter;
 
-            operation->key.implementation = (char*) my_malloc (90);
+            operation->key.implementation = (char*) my_malloc (BIG_BUFFER);
             sprintf(operation->key.implementation, "    ");
-            strncat(operation->key.implementation, attrlist->key.name, 76);
+            strcat(operation->key.implementation, attrlist->key.name);
             strcat(operation->key.implementation, " = value;");
 
             tmpname = strtoupperfirst(attrlist->key.name);
             sprintf(operation->key.attr.name, "set");
-            strncat(operation->key.attr.name, tmpname, 76);
+            strncat(operation->key.attr.name, tmpname, SMALL_BUFFER - 1);
             free(tmpname);
             operation->key.attr.isabstract = 0;
             operation->key.attr.isstatic = 0;
@@ -337,7 +337,7 @@ void make_javabean_methods(umlclass *myself) {
             myself->operations = insert_operation(operation, myself->operations);
 
             /* The GET or IS method */
-            operation = (umloplist) my_malloc (sizeof(umlopnode));
+            operation = NEW (umlopnode);
             operation->key.parameters = NULL;
             tmpname = strtoupperfirst(attrlist->key.name);
             if ( eq(attrlist->key.type, "boolean") ) {
@@ -345,12 +345,12 @@ void make_javabean_methods(umlclass *myself) {
             } else {
                 sprintf(operation->key.attr.name, "get");
             }
-            strncat(operation->key.attr.name, tmpname, 76);
+            strncat(operation->key.attr.name, tmpname, SMALL_BUFFER - 1);
             free(tmpname);
 
-            operation->key.implementation = (char*) my_malloc (90);
+            operation->key.implementation = (char*) my_malloc (BIG_BUFFER);
             sprintf(operation->key.implementation, "    return ");
-            strncat(operation->key.implementation, attrlist->key.name, 76);
+            strcat(operation->key.implementation, attrlist->key.name);
             strcat(operation->key.implementation, ";");
 
             operation->key.attr.isabstract = 0;
@@ -358,7 +358,7 @@ void make_javabean_methods(umlclass *myself) {
             operation->key.attr.isconstant = 0;
             operation->key.attr.visibility = '0';
             operation->key.attr.value[0] = 0;
-            strncpy(operation->key.attr.type, attrlist->key.type, SMALL_BUFFER);
+            strncpy(operation->key.attr.type, attrlist->key.type, SMALL_BUFFER - 1);
             operation->next = NULL;
 
             myself->operations = insert_operation(operation, myself->operations);
@@ -407,8 +407,8 @@ umlpackagelist parse_package(xmlNodePtr package) {
     xmlChar *attrname;
     //debug( 4, "parse_package %s", package->name );
 
-    listmyself = (umlpackagelist) my_malloc ( sizeof ( umlpackagenode));
-    myself = (umlpackage*) my_malloc(sizeof(umlpackage));
+    listmyself = NEW (umlpackagenode);
+    myself = NEW (umlpackage);
 
     myself->parent = NULL;
 
@@ -442,9 +442,10 @@ umlclasslist parse_class(xmlNodePtr class) {
     umlclasslist listmyself;
     umlclass *myself;
 
-    listmyself = (umlclasslist) my_malloc (sizeof (umlclassnode));
-    myself = (umlclass*) my_malloc (sizeof(umlclass));
+    listmyself = NEW (umlclassnode);
+    myself = NEW (umlclass);
     myself->package = NULL;
+    myself->isabstract = 0;
 
     listmyself->key = myself;
     listmyself->parents = NULL;
@@ -510,7 +511,7 @@ umlclasslist parse_class(xmlNodePtr class) {
 */
 void lolipop_implementation(umlclasslist classlist, xmlNodePtr object) {
     xmlNodePtr attribute;
-    xmlChar *id = NULL, *name = NULL;
+    xmlChar *id = NULL, *name = "";
     xmlChar *attrname;
     umlclasslist interface, implementator;
 
@@ -532,9 +533,9 @@ void lolipop_implementation(umlclasslist classlist, xmlNodePtr object) {
     }
     implementator = find(classlist, id);
     free(id);
-    if (implementator != NULL && strlen(name) > 2) {
-        interface = (umlclasslist) my_malloc (sizeof (umlclassnode));
-        interface->key = (umlclass *) my_malloc (sizeof(umlclass));
+    if (implementator != NULL && name != NULL && strlen(name) > 2) {
+        interface = NEW (umlclassnode);
+        interface->key = NEW (umlclass);
         interface->parents = NULL;
         interface->next = NULL;
         sprintf(interface->key->id, "00");
@@ -600,14 +601,12 @@ xmlNodePtr getNextObject(xmlNodePtr from) {
 
 umlclasslist parse_diagram(char *diafile) {
     xmlDocPtr ptr;
-    xmlChar *objtype, *attrtype, *tmptype, *tmptype2;
-    xmlChar *objid;
     xmlChar *end1 = NULL;
     xmlChar *end2 = NULL;
 
-    xmlNodePtr  object = NULL, attribute;
-    umlclasslist classlist, tmplist, endlist = NULL;
-    umlpackagelist packagelist, dummypcklist, tmppcklist, endpcklist = NULL;
+    xmlNodePtr object = NULL;
+    umlclasslist classlist = NULL, endlist = NULL;
+    umlpackagelist packagelist = NULL, dummypcklist, endpcklist = NULL;
 
     ptr = xmlParseFile(diafile);
 
@@ -619,18 +618,15 @@ umlclasslist parse_diagram(char *diafile) {
     /* we search for the first "object" node */
     recursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
 
-    classlist = NULL;
-    packagelist = NULL;
-
     while (object != NULL) {
-        objtype = xmlGetProp(object, "type");
+        xmlChar *objtype = xmlGetProp(object, "type");
         /* Here we have a Dia object */
-        if ( eq("UML - Class", objtype) ) {
+        if ( eq("UML - Class", objtype) || eq("Database - Table", objtype)) {
             /* Here we have a class definition */
-            tmplist = parse_class(object);
+            umlclasslist tmplist = parse_class(object);
             if (tmplist != NULL) {
                 /* We get the ID of the object here*/
-                objid = xmlGetProp(object, "id");
+                xmlChar *objid = xmlGetProp(object, "id");
                 sscanf(objid, "%79s", tmplist->key->id);
                 free(objid);
 
@@ -642,11 +638,11 @@ umlclasslist parse_diagram(char *diafile) {
                     endlist = tmplist;
                 }
             }
-        } else if ( eq("UML - LargePackage",objtype) || eq("UML - SmallPackage",objtype)) {
-            tmppcklist = parse_package(object);
+        } else if ( eq("UML - LargePackage", objtype) || eq("UML - SmallPackage", objtype) ) {
+            umlpackagelist tmppcklist = parse_package(object);
             if ( tmppcklist != NULL ) {
                 /* We get the ID of the object here*/
-                objid = xmlGetProp(object, "id");
+                xmlChar *objid = xmlGetProp(object, "id");
                 sscanf(objid, "%79s", tmppcklist->key->id);
                 free(objid);
             }
@@ -674,24 +670,22 @@ umlclasslist parse_diagram(char *diafile) {
     recursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
 
     while ( object != NULL ) {
-        objtype = xmlGetProp(object, "type");
+        xmlChar *objtype = xmlGetProp(object, "type");
         if ( eq("UML - Association", objtype) ) {
-            char * name = NULL;
+            char *name = NULL, *name_a = NULL, *name_b = NULL;
+            char *multiplicity_a = NULL, *multiplicity_b = NULL;
             char direction = 0;
             char composite = 0;
-            attribute = object->xmlChildrenNode;
+            xmlNodePtr attribute = object->xmlChildrenNode;
 
-            while ( attribute != NULL) {
-                attrtype = xmlGetProp(attribute, "name");
+            while (attribute != NULL) {
+                xmlChar *attrtype = xmlGetProp(attribute, "name");
 
                 if (attrtype != NULL) {
-                    if ( eq("name", attrtype) && attribute->xmlChildrenNode->xmlChildrenNode ) {
-                        name = attribute -> xmlChildrenNode -> xmlChildrenNode -> content;
-                    }
-
-                    else if ( ! strcmp("direction", attrtype) ) {
-                        tmptype = xmlGetProp(attribute -> xmlChildrenNode, "val");
-                        if ( !strcmp("0", tmptype) ) {
+                    xmlNodePtr child = attribute->xmlChildrenNode;
+                    if ( eq("direction", attrtype) ) {
+                        xmlChar *tmptype = xmlGetProp(child, "val");
+                        if ( eq(tmptype, "0") ) {
                             direction = 1;
                         }
                         else {
@@ -699,10 +693,9 @@ umlclasslist parse_diagram(char *diafile) {
                         }
                         free(tmptype);
                     }
-
-                    else if ( ! strcmp("assoc_type", attrtype) ) {
-                        tmptype = xmlGetProp(attribute -> xmlChildrenNode, "val");
-                        if ( !strcmp("1", tmptype) ) {
+                    else if ( eq("assoc_type", attrtype) ) {
+                        xmlChar *tmptype = xmlGetProp(child, "val");
+                        if ( eq(tmptype, "1") ) {
                             composite = 0;
                         }
                         else {
@@ -710,33 +703,95 @@ umlclasslist parse_diagram(char *diafile) {
                         }
                         free(tmptype);
                     }
+                    else if ( child->xmlChildrenNode ) {
+                        xmlNodePtr grandchild = child->xmlChildrenNode;
+                        if ( eq(attrtype, "name") ) {
+                            name = grandchild->content;
+                        }
+                        else if ( eq(attrtype, "role_a") ) {
+                            name_a = grandchild->content;
+                        }
+                        else if ( eq(attrtype, "role_b") ) {
+                            name_b = grandchild->content;
+                        }
+                        else if ( eq(attrtype, "multipicity_a") ) {
+                            multiplicity_a = grandchild->content;
+                        }
+                        else if ( eq(attrtype, "multipicity_b") ) {
+                            multiplicity_b = grandchild->content;
+                        } 
+                        else if ( eq(attrtype, "ends") ) {
+                            if ( eq(child->name, "composite") ) {
+				while (grandchild) {
+                                    xmlNodePtr ggchild = grandchild->xmlChildrenNode;
+				    if (ggchild->xmlChildrenNode) {
+                                        attrtype = xmlGetProp(grandchild, "name");
+                                        if ( eq(attrtype, "role") ) {
+                                            name_a = ggchild->xmlChildrenNode->content;
+                                        }
+                                        else if ( eq(attrtype, "multiplicity") ) {
+                                            multiplicity_a = ggchild->xmlChildrenNode->content;
+                                        }
+                                        else if ( eq(attrtype, "aggregate") ) {
+                                            /* todo */
+                                        }
+                                    }
+				    grandchild = grandchild->next;
+                                }
+                            }
+                            child = child->next;
+                            if ( child != NULL && eq(child->name, "composite") ) {
+                                grandchild = child->xmlChildrenNode;
+				while (grandchild) {
+                                    xmlNodePtr ggchild = grandchild->xmlChildrenNode;
+				    if (ggchild->xmlChildrenNode) {
+                                        attrtype = xmlGetProp(grandchild, "name");
+                                        if ( eq(attrtype, "role") ) {
+                                            name_b = ggchild->xmlChildrenNode->content;
+                                        }
+                                        else if ( eq(attrtype, "multiplicity") ) {
+                                            multiplicity_b = ggchild->xmlChildrenNode->content;
+                                        }
+                                        else if ( eq(attrtype, "aggregate") ) {
+                                            /* todo */
+                                        }
+                                    }
+				    grandchild = grandchild->next;
+                                }
+                            }
+                        }
+                    }
+                    free(attrtype);
                 }
-
-                if ( ! strcmp("connections", attribute -> name) ) {
+                else if ( eq(attribute->name, "connections") ) {
                     end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
                     end2 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
                 }
 
-                free(attrtype);
                 attribute = attribute->next;
             }
 
             if (end1 != NULL && end2 != NULL) {
+                char *thisname = name;
                 if (direction == 1) {
-                    associate(classlist, name, composite, end1, end2);
+                    if (thisname == NULL || !*thisname || eq("##", thisname))
+                        thisname = name_a;
+                    associate(classlist, thisname, composite, end1, end2, multiplicity_a);
                 } else {
-                    associate(classlist, name, composite, end2, end1);
+                    if (thisname == NULL || !*thisname || eq("##", thisname))
+                        thisname = name_b;
+                    associate(classlist, thisname, composite, end2, end1, multiplicity_b);
                 }
                 free(end1);
                 free(end2);
             }
 
         } else if ( eq("UML - Dependency", objtype) ) {
-            attribute = object->xmlChildrenNode;
+            xmlNodePtr attribute = object->xmlChildrenNode;
             while ( attribute != NULL ) {
                 if ( eq("connections", attribute->name) ) {
-                    end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
-                    end2 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
+                    end1 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
+                    end2 = xmlGetProp(attribute->xmlChildrenNode, "to");
                     make_depend(classlist, end1, end2);
                     free(end1);
                     free(end2);
@@ -744,7 +799,7 @@ umlclasslist parse_diagram(char *diafile) {
                 attribute = attribute->next;
             }
         } else if ( eq("UML - Realizes", objtype) ) {
-            attribute = object->xmlChildrenNode;
+            xmlNodePtr attribute = object->xmlChildrenNode;
             while ( attribute != NULL ) {
                 if ( eq("connections", attribute->name) ) {
                     end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
@@ -767,9 +822,9 @@ umlclasslist parse_diagram(char *diafile) {
        implementations. generate_code_java relies on this. */
     recursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
     while ( object != NULL ) {
-        objtype = xmlGetProp(object, "type");
+        xmlChar *objtype = xmlGetProp(object, "type");
         if ( eq("UML - Generalization", objtype) ) {
-            attribute = object->xmlChildrenNode;
+            xmlNodePtr attribute = object->xmlChildrenNode;
             while ( attribute != NULL ) {
                 if ( eq("connections", attribute->name) ) {
                     end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
@@ -795,9 +850,9 @@ umlclasslist parse_diagram(char *diafile) {
     /* Build the relationships between packages */
     dummypcklist = packagelist;
     while ( dummypcklist != NULL ) {
-        tmppcklist = packagelist;
+        umlpackagelist tmppcklist = packagelist;
         while ( tmppcklist != NULL ) {
-            if ( is_inside(&dummypcklist->key->geom,&tmppcklist->key->geom) ) {
+            if ( is_inside(&dummypcklist->key->geom, &tmppcklist->key->geom) ) {
                 if ( tmppcklist->key->parent == NULL ) {
                     tmppcklist->key->parent = dummypcklist->key;
                 } else {
@@ -814,7 +869,7 @@ umlclasslist parse_diagram(char *diafile) {
     /* Associate packages to classes */
     dummypcklist = packagelist;
     while ( dummypcklist != NULL ) {
-        tmplist = classlist;
+        umlclasslist tmplist = classlist;
         while ( tmplist != NULL ) {
             if ( is_inside(&dummypcklist->key->geom,&tmplist->key->geom) ) {
                 if ( tmplist->key->package == NULL ) {
